#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import logging
import os
from io import BytesIO
from xml.dom import minidom
from xml.sax.saxutils import escape
from xmind2testcase import const
from xmind2testcase.parser import config
from xmind2testcase.utils import get_xmind_testsuites, get_absolute_path
from xml.etree.ElementTree import Element, SubElement, ElementTree, Comment

"""
Convert XMind fie to TestLink testcase xml file 
"""


def xmind_to_testlink_xml_file(xmind_file, is_all_sheet=True):
    """Convert a XMind sheet to a testlink xml file"""
    xmind_file = get_absolute_path(xmind_file)
    logging.info('Start converting XMind file(%s) to testlink file...', xmind_file)
    testsuites = get_xmind_testsuites(xmind_file)
    if not is_all_sheet and testsuites:
        testsuites = [testsuites[0]]

    xml_content = testsuites_to_xml_content(testsuites)
    testlink_xml_file = xmind_file[:-6] + '.xml'

    if os.path.exists(testlink_xml_file):
        logging.info('the testlink xml file already exists, return it directly: %s', testlink_xml_file)
        return testlink_xml_file

    with open(testlink_xml_file, 'w', encoding='utf-8') as f:
        pretty_content = minidom.parseString(xml_content).toprettyxml(indent='\t')
        f.write(pretty_content)
        logging.info('convert XMind file(%s) to a testlink xml file(%s) successfully!', xmind_file, testlink_xml_file)

    return testlink_xml_file


def testsuites_to_xml_content(testsuites):
    """Convert the testsuites to testlink xml file format"""
    root_element = Element(const.TAG_TESTSUITE)
    # setting the root suite's name attribute, that will generate a new testsuite folder on testlink
    # root_element.set(const.ATTR_NMAE, testsuite.name)

    for testsuite in testsuites:
        suite_element = SubElement(root_element, const.TAG_TESTSUITE)
        suite_element.set(const.ATTR_NMAE, testsuite.name)
        gen_text_element(suite_element, const.TAG_DETAILS, testsuite.details)

        for sub_suite in testsuite.sub_suites:
            if is_should_skip(sub_suite.name):
                continue
            sub_suite_element = SubElement(suite_element, const.TAG_TESTSUITE)
            sub_suite_element.set(const.ATTR_NMAE, sub_suite.name)
            gen_text_element(sub_suite_element, const.TAG_DETAILS, sub_suite.details)
            gen_testcase_element(sub_suite_element, sub_suite)

    testlink = ElementTree(root_element)
    content_stream = BytesIO()
    testlink.write(content_stream, encoding='utf-8', xml_declaration=True)
    return content_stream.getvalue()


def gen_testcase_element(suite_element, suite):
    for testcase in suite.testcase_list:

        if is_should_skip(testcase.name):
            continue

        testcase_elment = SubElement(suite_element, const.TAG_TESTCASE)
        testcase_elment.set(const.ATTR_NMAE, testcase.name)

        gen_text_element(testcase_elment, const.TAG_VERSION, str(testcase.version))
        gen_text_element(testcase_elment, const.TAG_SUMMARY, testcase.summary)
        gen_text_element(testcase_elment, const.TAG_PRECONDITIONS, testcase.preconditions)
        gen_text_element(testcase_elment, const.TAG_EXECUTION_TYPE, _convert_execution_type(testcase.execution_type))
        gen_text_element(testcase_elment, const.TAG_IMPORTANCE, _convert_importance(testcase.importance))

        estimated_exec_duration_element = SubElement(testcase_elment, const.TAG_ESTIMATED_EXEC_DURATION)
        estimated_exec_duration_element.text = str(testcase.estimated_exec_duration)

        status = SubElement(testcase_elment, const.TAG_STATUS)
        status.text = str(testcase.status) if testcase.status in (1, 2, 3, 4, 5, 6, 7) else '7'

        gen_steps_element(testcase_elment, testcase)


def gen_steps_element(testcase_element, testcase):
    if testcase.steps:
        steps_element = SubElement(testcase_element, const.TAG_STEPS)

        for step in testcase.steps:

            if is_should_skip(step.actions):
                continue

            step_element = SubElement(steps_element, const.TAG_STEP)
            gen_text_element(step_element, const.TAG_STEP_NUMBER, str(step.step_number))
            gen_text_element(step_element, const.TAG_ACTIONS, step.actions)
            gen_text_element(step_element, const.TAG_EXPECTEDRESULTS, step.expectedresults)
            gen_text_element(step_element, const.TAG_EXECUTION_TYPE, _convert_execution_type(step.execution_type))


def gen_text_element(parent_element, tag_name, content):
    """generate an element's text conent: <![CDATA[text]]>"""
    if is_should_parse(content):
        child_element = SubElement(parent_element, tag_name)
        element_set_text(child_element, content)


def element_set_text(element, content):
    # retain html tags in content
    content = escape(content, entities={'\r\n': '<br />'})
    # replace new line for *nix system
    content = content.replace('\n', '<br />')
    # add the line break in source to make it readable
    content = content.replace('<br />', '<br />\n')

    # add CDATA for a element
    element.append(Comment(' --><![CDATA[' + content.replace(']]>', ']]]]><![CDATA[>') + ']]> <!-- '))


def is_should_parse(content):
    """An element that has a string content and doesn't start with exclamation mark should be parsing"""
    return isinstance(content, str) and content.strip() != '' and not content[0] in config['ignore_char']


def is_should_skip(content):
    """A testsuite/testcase/teststep should be skip: 1、content is empty; 2、starts with config.ignore_char"""
    return content is None or \
        not isinstance(content, str) or \
        content.strip() == '' or \
        content[0] in config['ignore_char']


def _convert_execution_type(value):
    if value in (1, '手动', '手工', 'manual', 'Manual'):
        return '1'
    elif value in (2, '自动', '自动化', '自动的', 'Automate', 'Automated', 'Automation', 'automate', 'automated', 'automation'):
        return '2'
    else:
        return '1'


def _convert_importance(value):
    mapping = {1: '3', 2: '2', 3: '1'}
    if value in mapping.keys():
        return mapping[value]
    else:
        return '2'


if __name__ == '__main__':
    xmind_file = '../docs/xmind_testcase_template.xmind'
    testlink_xml_file = xmind_to_testlink_xml_file(xmind_file)
    print('Convert XMind file to testlink xml file successfully: %s', testlink_xml_file)


#!/usr/bin/env python
# _*_ coding:utf-8 _*_


TAG_XML = 'xml'

TAG_TESTSUITE = 'testsuite'
TAG_DETAILS = 'details'

TAG_TESTCASE = 'testcase'
TAG_VERSION = 'version'
TAG_SUMMARY = 'summary'
TAG_PRECONDITIONS = 'preconditions'
TAG_IMPORTANCE = 'importance'
TAG_ESTIMATED_EXEC_DURATION = 'estimated_exec_duration'
TAG_STATUS = 'status'
TAG_IS_OPEN = 'is_open'
TAG_ACTIVE = 'active'
TAG_STEPS = 'steps'
TAG_STEP = 'step'
TAG_STEP_NUMBER = 'step_number'
TAG_ACTIONS = 'actions'
TAG_EXPECTEDRESULTS = 'expectedresults'
TAG_EXECUTION_TYPE = 'execution_type'

ATTR_NMAE = 'name'
ATTR_ID = 'id'
ATTR_INTERNALID = 'internalid'



#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import csv
import logging
import os
from xmind2testcase.utils import get_xmind_testcase_list, get_absolute_path
import re
import pandas as pd
"""
Convert XMind fie to TestRail testcase csv file 

"""


def xmind_to_testrail_csv_file(xmind_file):
    """Convert XMind file to a testrail csv file"""
    xmind_file = get_absolute_path(xmind_file)   # 获取xmind的绝对路径
    logging.info('Start converting XMind file(%s) to testrail file...', xmind_file)   # 打印日志信息
    testcases = get_xmind_testcase_list(xmind_file)   # 获取testcase
    # print(testcases)
    # fileheader = ["Section", "Title", "优先级", "版本", "前置条件", "测试步骤", "预期结果"]   # csv表格标题
    fileheader = ["用例名称", "所属模块", "用例等级", "前提条件", "用例步骤", "预期结果"]   # csv表格标题
    testrail_testcase_rows = [fileheader]
    for testcase in testcases:
        row = gen_a_testcase_row(testcase)
        testrail_testcase_rows.append(row)

    testrail_file = xmind_file[:-6] + '.csv'
    if os.path.exists(testrail_file):
        os.remove(testrail_file)
        # logging.info('The testrail csv file already exists, return it directly: %s', testrail_file)
        # return testrail_file

    with open(testrail_file, 'w', encoding='utf8', newline='') as f:
        writer = csv.writer(f)
        writer.writerows(testrail_testcase_rows)
        logging.info('Convert XMind file(%s) to a testrail csv file(%s) successfully!', xmind_file, testrail_file)

    return testrail_file

def xmind_to_testrail_xlsx_file(xmind_file):
    """Convert XMind file to a testrail xlsx file"""
    xmind_file = get_absolute_path(xmind_file)   # 获取xmind的绝对路径
    logging.info('Start converting XMind file(%s) to testrail file...', xmind_file)   # 打印日志信息
    testcases = get_xmind_testcase_list(xmind_file)   # 获取testcase
    # print(testcases)
    fileheader = ["用例名称", "所属产品", "所属模块", "用例等级", "用例类型", "前提条件", "用例步骤", "预期结果"]   # csv表格标题
    testrail_testcase_rows = [fileheader]
    for testcase in testcases:
        row = gen_a_testcase_row(testcase)
        testrail_testcase_rows.append(row)

    testrail_file = xmind_file[:-6] + '.xlsx'
    if os.path.exists(testrail_file):
        os.remove(testrail_file)
        # logging.info('The testrail csv file already exists, return it directly: %s', testrail_file)
        # return testrail_file
        df = pd.DataFrame(data=testrail_testcase_rows, columns=fileheader)  # 构造数据
        df.to_excel(testrail_file, index=False)  # 写入文件，设置不需要索引
        logging.info('Convert XMind file(%s) to a testrail xlsx file(%s) successfully!', xmind_file, testrail_file)
    return testrail_file




def gen_a_testcase_row(testcase_dict):
    case_title_list = testcase_dict['name'].split()   # 用例title
    case_title = case_title_list[-1]
    del(case_title_list[-1])
    case_section = '/' + gen_case_section(testcase_dict['suite'])  # 获取Section
    # print("case_title：" + case_title)
    # print(case_title_list)
    for i in case_title_list:
        case_section = case_section + '/'+i

    # print("case_section:" + case_section)

    case_precontion = testcase_dict['preconditions']   # 前置条件
    case_step, case_expected_result = gen_case_step_and_expected_result(testcase_dict['steps'])   # 测试步骤和预期结果
    # case_keyword = ''
    case_priority = gen_case_priority(testcase_dict['importance'])  # 优先级
    case_type = gen_case_type(testcase_dict['execution_type'])  # 用例类型
    # print("case_type:" + case_type)
    case_version = testcase_dict['version']
    case_product = testcase_dict['product']
    # print(testcase_dict)
    # case_apply_phase = '迭代测试'
    row = [case_title, case_product, case_section,  case_priority, case_type, case_precontion, case_step, case_expected_result]
    return row


def gen_case_section(section_name):
    if section_name:
        section_name = section_name.replace('（', '(')
        section_name = section_name.replace('）', ')')
    else:
        section_name = '/'
    return section_name


def gen_case_step_and_expected_result(steps):
    case_step = ''
    case_expected_result = ''

    for step_dict in steps:
        actions_num = re.compile(r'\d+[、.]').findall(step_dict['actions'])
        expectedresults_num = re.compile(r'\d+[、.]').findall(step_dict['expectedresults'])
        if actions_num == []:
            case_step += str(step_dict['step_number']) + '、' + step_dict['actions'].replace('\n', '').strip() + '\n'
        else:
            case_step += step_dict['actions'].replace('\n', '').strip() + '\n'
        if expectedresults_num == []:
            case_expected_result += str(step_dict['step_number']) + '.0、 ' + \
                                    step_dict['expectedresults'].replace('\n', '').strip() + '\n' \
                if step_dict.get('expectedresults', '') else ''
        else:
            case_expected_result += step_dict['expectedresults'].replace('\n', '').strip() + '\n' \
                if step_dict.get('expectedresults', '') else ''

    return case_step, case_expected_result


def gen_case_priority(priority):
    mapping = {1: 'L0', 2: 'L1', 3: 'L2', 4: 'L3'}
    if priority in mapping.keys():
        return mapping[priority]
    else:
        return 'L2'


def gen_case_type(case_type):
    #“是否实现自动化” 1为否，2为是
    mapping = {1: '功能', 2: '性能', 3: '可靠性', 4: '安全', 5: '兼容性', 6: '用户体验', 7: '可运维'}
    if case_type in mapping.keys():
        return mapping[case_type]
    else:
        return '功能'


if __name__ == '__main__':
    # xmind_file = '../docs/testrail_testcase_template.xmind'
    xmind_file = 'C:/Users/YYS/Desktop/草帽云V1.5.0测试用例.xmind'
    # get_xmind_testcase_list(xmind_file)
    testrail_csv_file = xmind_to_testrail_xlsx_file(xmind_file)
    # print('Conver the xmind file to a testrail csv file succssfully: %s', testrail_csv_file)


import tkinter
import pygeoip


class FindLocation(object):
    def __init__(self):
        self.gi = pygeoip.GeoIP("./GeoLiteCity.dat")
        # 创建主窗口,用于容纳其它组件
        self.root = tkinter.Tk()
        # 给主窗口设置标题内容
        self.root.title("全球定位ip位置(离线版)")
        # 创建一个输入框,并设置尺寸
        self.ip_input = tkinter.Entry(self.root, width=30)

        # 创建一个回显列表
        self.display_info = tkinter.Listbox(self.root, width=50)

        # 创建一个查询结果的按钮
        self.result_button = tkinter.Button(self.root, command=self.find_position, text="查询")

    # 完成布局
    def gui_arrang(self):
        self.ip_input.pack()
        self.display_info.pack()
        self.result_button.pack()

    # 根据ip查找地理位置
    def find_position(self):
        # 获取输入信息
        self.ip_addr = self.ip_input.get()
        aim = self.gi.record_by_name(self.ip_addr)
        # 为了避免非法值,导致程序崩溃,有兴趣可以用正则写一下具体的规则,我为了便于新手理解,减少代码量,就直接粗放的过滤了
        try:

            # 获取目标城市
            city = aim["city"]
            # 获取目标国家
            country = aim["country_name"]
            # 获取目标地区
            region_code = aim["region_code"]
            # 获取目标经度
            longitude = aim["longitude"]
            # 获取目标纬度
            latitude = aim["latitude"]
        except:
            pass

        # 创建临时列表
        the_ip_info = ["所在纬度:" + str(latitude), "所在经度:" + str(longitude), "地域代号:" + str(region_code),
                       "所在城市:" + str(city), "所在国家或地区:" + str(country), "需要查询的ip:" + str(self.ip_addr)]
        # 清空回显列表可见部分,类似clear命令
        for item in range(10):
            self.display_info.insert(0, "")

        # 为回显列表赋值
        for item in the_ip_info:
            self.display_info.insert(0, item)
        # 这里的返回值,没啥用,就是为了好看
        return the_ip_info


def main():
    # 初始化对象
    FL = FindLocation()
    # 进行布局
    FL.gui_arrang()
    # 主程序执行
    tkinter.mainloop()
    pass


if __name__ == "__main__":
    main()


#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import csv
import logging
import os
from xmind2testcase.utils import get_xmind_testcase_list, get_absolute_path

"""
Convert XMind fie to Zentao testcase csv file 

Zentao official document about import CSV testcase file: https://www.zentao.net/book/zentaopmshelp/243.mhtml 
"""


def xmind_to_zentao_csv_file(xmind_file):
    """Convert XMind file to a zentao csv file"""
    xmind_file = get_absolute_path(xmind_file)
    logging.info('Start converting XMind file(%s) to zentao file...', xmind_file)
    testcases = get_xmind_testcase_list(xmind_file)

    fileheader = ["所属模块", "用例标题", "前置条件", "步骤", "预期", "关键词", "优先级", "用例类型", "适用阶段"]
    zentao_testcase_rows = [fileheader]
    for testcase in testcases:
        row = gen_a_testcase_row(testcase)
        zentao_testcase_rows.append(row)

    zentao_file = xmind_file[:-6] + '.csv'
    if os.path.exists(zentao_file):
        os.remove(zentao_file)
        # logging.info('The zentao csv file already exists, return it directly: %s', zentao_file)
        # return zentao_file

    with open(zentao_file, 'w', encoding='utf8') as f:
        writer = csv.writer(f)
        writer.writerows(zentao_testcase_rows)
        logging.info('Convert XMind file(%s) to a zentao csv file(%s) successfully!', xmind_file, zentao_file)

    return zentao_file


def gen_a_testcase_row(testcase_dict):
    case_module = gen_case_module(testcase_dict['suite'])
    case_title = testcase_dict['name']
    case_precontion = testcase_dict['preconditions']
    case_step, case_expected_result = gen_case_step_and_expected_result(testcase_dict['steps'])
    case_keyword = ''
    case_priority = gen_case_priority(testcase_dict['importance'])
    case_type = gen_case_type(testcase_dict['execution_type'])
    case_apply_phase = '迭代测试'
    row = [case_module, case_title, case_precontion, case_step, case_expected_result, case_keyword, case_priority, case_type, case_apply_phase]
    return row


def gen_case_module(module_name):
    if module_name:
        module_name = module_name.replace('（', '(')
        module_name = module_name.replace('）', ')')
    else:
        module_name = '/'
    return module_name


def gen_case_step_and_expected_result(steps):
    case_step = ''
    case_expected_result = ''

    for step_dict in steps:
        case_step += str(step_dict['step_number']) + '. ' + step_dict['actions'].replace('\n', '').strip() + '\n'
        case_expected_result += str(step_dict['step_number']) + '. ' + \
            step_dict['expectedresults'].replace('\n', '').strip() + '\n' \
            if step_dict.get('expectedresults', '') else ''

    return case_step, case_expected_result


def gen_case_priority(priority):
    mapping = {1: '高', 2: '中', 3: '低'}
    if priority in mapping.keys():
        return mapping[priority]
    else:
        return '中'


def gen_case_type(case_type):
    mapping = {1: '手动', 2: '自动'}
    if case_type in mapping.keys():
        return mapping[case_type]
    else:
        return '手动'


if __name__ == '__main__':
    xmind_file = '../docs/zentao_testcase_template.xmind'
    zentao_csv_file = xmind_to_zentao_csv_file(xmind_file)
    print('Conver the xmind file to a zentao csv file succssfully: %s', zentao_csv_file)


#!/usr/bin/env python
# _*_ coding:utf-8 _*_

"""
testlink.testlink
"""


class TestSuite(object):

    def __init__(self, name='', details='', testcase_list=None, sub_suites=None, statistics=None):
        """
        TestSuite
        :param name: test suite name
        :param details: test suite detail infomation
        :param testcase_list: test case list
        :param sub_suites: sub test suite list
        :param statistics: testsuite statistics info {'case_num': 0, 'non_execution': 0, 'pass': 0, 'failed': 0, 'blocked': 0, 'skipped': 0}
        """
        self.name = name
        self.details = details
        self.testcase_list = testcase_list
        self.sub_suites = sub_suites
        self.statistics = statistics

    def to_dict(self):
        data = {
            'name': self.name,
            'details': self.details,
            'testcase_list': [],
            'sub_suites': []
        }

        if self.sub_suites:
            for suite in self.sub_suites:
                data['sub_suites'].append(suite.to_dict())

        if self.testcase_list:
            for case in self.testcase_list:
                data['testcase_list'].append(case.to_dict())

        if self.statistics:
            data['statistics'] = self.statistics

        return data


class TestCase(object):

    def __init__(self, name='', version='V1.0.0', summary='', preconditions='', execution_type=1, importance=2, estimated_exec_duration=3, status=7, result=0, steps=None):
        """
        TestCase
        :param name: test case name
        :param version: test case version infomation
        :param summary: test case summary infomation
        :param preconditions: test case pre condition
        :param execution_type: manual:1 or automate:2
        :param importance: high:1, middle:2, low:3
        :param estimated_exec_duration: estimated execution duration
        :param status: draft:1, ready ro review:2, review in progress:3, rework:4, obsolete:5, future:6, final:7
        :param result: non-execution:0, pass:1, failed:2, blocked:3, skipped:4
        :param steps: test case step list
        """
        self.name = name
        self.version = version
        self.summary = summary
        self.preconditions = preconditions
        self.execution_type = execution_type
        self.importance = importance
        self.estimated_exec_duration = estimated_exec_duration
        self.status = status
        self.result = result
        self.steps = steps

    def to_dict(self):
        data = {
            'name': self.name,
            'version': self.version,  # TODO(devin): get version content
            'summary': self.summary,
            'preconditions': self.preconditions,
            'execution_type': self.execution_type,
            'importance': self.importance,
            'estimated_exec_duration': self.estimated_exec_duration,  # TODO(devin): get estimated content
            'status': self.status,  # TODO(devin): get status content
            'result': self.result,
            'steps': []
        }

        if self.steps:
            for step in self.steps:
                data['steps'].append(step.to_dict())

        return data


class TestStep(object):

    def __init__(self, step_number=1, actions='', expectedresults='', execution_type=1, result=0):
        """
        TestStep
        :param step_number: test step number
        :param actions: test step actions
        :param expectedresults: test step expected results
        :param execution_type: test step execution type
        :param result: non-execution:0, pass:1, failed:2, blocked:3, skipped:4
        """
        self.step_number = step_number
        self.actions = actions
        self.expectedresults = expectedresults
        self.execution_type = execution_type  # TODO(devin): get execution type content
        self.result = result

    def to_dict(self):
        data = {
            'step_number': self.step_number,
            'actions': self.actions,
            'expectedresults': self.expectedresults,
            'execution_type': self.execution_type,
            'result': self.result
        }

        return data

